@@ -14,6 +14,7 @@
 #include <xf86drm.h>
 #include <xf86drmMode.h>
 
+#include "base64.h"
 #include "drm_info.h"
 #include "tables.h"
 
@@ -239,6 +240,15 @@ static struct json_object *device_info(int fd)
 		json_object_object_add(bus_data_obj, "fullname",
 			json_object_new_string(host1x_bus->fullname));
 		break;
+#if HAVE_FAUX_BUS
+	case DRM_BUS_FAUX:;
+		drmFauxBusInfo *faux_bus = dev->businfo.faux;
+
+		bus_data_obj = json_object_new_object();
+		json_object_object_add(bus_data_obj, "name",
+			json_object_new_string(faux_bus->name));
+		break;
+#endif
 	}
 	json_object_object_add(obj, "device_data", device_data_obj);
 	json_object_object_add(obj, "bus_data", bus_data_obj);
@@ -517,6 +527,50 @@ static struct json_object *fb_info(int fd, uint32_t id)
 	return obj;
 }
 
+static struct json_object *edid_info(int fd, uint32_t id)
+{
+	drmModePropertyBlobRes *blob = drmModeGetPropertyBlob(fd, id);
+	if (!blob) {
+		perror("drmModeGetPropertyBlob");
+		return NULL;
+	}
+
+	char *b64 = base64_encode(blob->data, blob->length);
+	drmModeFreePropertyBlob(blob);
+
+	struct json_object *value_obj = json_object_new_string(b64);
+	free(b64);
+	return value_obj;
+}
+
+static struct json_object *tile_info(int fd, uint32_t id)
+{
+	drmModePropertyBlobRes *blob = drmModeGetPropertyBlob(fd, id);
+	if (!blob) {
+		perror("drmModeGetPropertyBlob");
+		return NULL;
+	}
+
+	int group_id, is_single_monitor, h_num, v_num, h_loc, v_loc, h_size, v_size;
+	int n = sscanf(blob->data, "%d:%d:%d:%d:%d:%d:%d:%d", &group_id, &is_single_monitor,
+		&h_num, &v_num, &h_loc, &v_loc, &h_size, &v_size);
+	drmModeFreePropertyBlob(blob);
+	if (n != 8) {
+		fprintf(stderr, "Malformed TILE property for blob %"PRIu32"\n", id);
+		return NULL;
+	}
+
+	struct json_object *obj = json_object_new_object();
+	json_object_object_add(obj, "group_id", json_object_new_int(group_id));
+	json_object_object_add(obj, "is_single_monitor", json_object_new_boolean(is_single_monitor));
+	json_object_object_add(obj, "h_num", json_object_new_int(h_num));
+	json_object_object_add(obj, "v_num", json_object_new_int(v_num));
+	json_object_object_add(obj, "h_loc", json_object_new_int(h_loc));
+	json_object_object_add(obj, "v_loc", json_object_new_int(v_loc));
+	json_object_object_add(obj, "h_size", json_object_new_int(h_size));
+	json_object_object_add(obj, "v_size", json_object_new_int(v_size));
+	return obj;
+}
 
 static struct json_object *properties_info(int fd, uint32_t id, uint32_t type)
 {
@@ -598,10 +652,6 @@ static struct json_object *properties_info(int fd, uint32_t id, uint32_t type)
 		case DRM_MODE_PROP_OBJECT:
 			value_obj = json_object_new_uint64(value);
 			break;
-		case DRM_MODE_PROP_BLOB:
-			// TODO: base64-encode blob contents
-			value_obj = NULL;
-			break;
 		case DRM_MODE_PROP_SIGNED_RANGE:
 			value_obj = json_object_new_int64((int64_t)value);
 			break;
@@ -611,7 +661,7 @@ static struct json_object *properties_info(int fd, uint32_t id, uint32_t type)
 		struct json_object *data_obj = NULL;
 		switch (type) {
 		case DRM_MODE_PROP_BLOB:
-			if (!value) {
+			if (!value && strcmp(prop->name, "TILE") != 0) {
 				break;
 			}
 			if (strcmp(prop->name, "IN_FORMATS") == 0) {
@@ -628,6 +678,10 @@ static struct json_object *properties_info(int fd, uint32_t id, uint32_t type)
 				data_obj = hdr_output_metadata_info(fd, value);
 			} else if (strcmp(prop->name, "SIZE_HINTS") == 0) {
 				data_obj = size_hints_info(fd, value);
+			} else if (strcmp(prop->name, "EDID") == 0) {
+				data_obj = edid_info(fd, value);
+			} else if (strcmp(prop->name, "TILE") == 0) {
+				data_obj = tile_info(fd, value);
 			}
 			break;
 		case DRM_MODE_PROP_RANGE:
