@@ -7,10 +7,15 @@
 #include <xf86drm.h>
 #include <xf86drmMode.h>
 
-#ifdef HAVE_LIBPCI
+#if HAVE_LIBPCI
 #include <pci/pci.h>
 #endif
 
+#if HAVE_LIBDISPLAY_INFO
+#include <libdisplay-info/info.h>
+#endif
+
+#include "base64.h"
 #include "drm_info.h"
 #include "modifiers.h"
 #include "tables.h"
@@ -95,6 +100,9 @@ static const char *bustype_str(int type)
 	case DRM_BUS_USB:      return "USB";
 	case DRM_BUS_PLATFORM: return "platform";
 	case DRM_BUS_HOST1X:   return "host1x";
+#if HAVE_FAUX_BUS
+	case DRM_BUS_FAUX:     return "faux";
+#endif
 	default:               return "unknown";
 	}
 }
@@ -131,48 +139,79 @@ static void print_device(struct json_object *obj)
 
 	int available_nodes = get_object_object_uint64(obj, "available_nodes");
 	int bus_type = get_object_object_uint64(obj, "bus_type");
-	struct json_object *data_obj = json_object_object_get(obj, "device_data");
-	struct json_object *compatible_arr = NULL;
+	struct json_object *device_data_obj = json_object_object_get(obj, "device_data");
+	struct json_object *bus_data_obj = json_object_object_get(obj, "bus_data");
+	struct json_object *compatible_arr;
+	const char *fullname;
 
 	printf(L_VAL "Device: %s", bustype_str(bus_type));
 	switch (bus_type) {
 	case DRM_BUS_PCI:;
-		uint16_t pci_vendor = get_object_object_uint64(data_obj, "vendor");
-		uint16_t pci_device = get_object_object_uint64(data_obj, "device");
+		uint16_t pci_vendor = get_object_object_uint64(device_data_obj, "vendor");
+		uint16_t pci_device = get_object_object_uint64(device_data_obj, "device");
 		printf(" %04x:%04x", pci_vendor, pci_device);
-#ifdef HAVE_LIBPCI
+#if HAVE_LIBPCI
 		struct pci_access *pci = pci_alloc();
 		pci_init(pci);
-		char name[512];
-		if (pci_lookup_name(pci, name, sizeof(name),
+		char pci_name[512];
+		if (pci_lookup_name(pci, pci_name, sizeof(pci_name),
 				PCI_LOOKUP_VENDOR | PCI_LOOKUP_DEVICE,
 				pci_vendor, pci_device)) {
-			printf(" %s", name);
+			printf(" %s", pci_name);
 		}
 		pci_cleanup(pci);
 #endif
+		printf("\n");
+
+		uint16_t pci_bus_domain = get_object_object_uint64(bus_data_obj, "domain");
+		uint8_t pci_bus = get_object_object_uint64(bus_data_obj, "bus");
+		uint8_t pci_bus_slot = get_object_object_uint64(bus_data_obj, "slot");
+		uint8_t pci_bus_function = get_object_object_uint64(bus_data_obj, "function");
+		printf(L_LINE L_VAL "Bus: %04x:%02x:%02x.%x\n",
+			pci_bus_domain, pci_bus, pci_bus_slot, pci_bus_function);
 		break;
 	case DRM_BUS_USB:;
-		uint16_t usb_vendor = get_object_object_uint64(data_obj, "vendor");
-		uint16_t usb_product = get_object_object_uint64(data_obj, "product");
-		printf(" %04x:%04x", usb_vendor, usb_product);
+		uint16_t usb_vendor = get_object_object_uint64(device_data_obj, "vendor");
+		uint16_t usb_product = get_object_object_uint64(device_data_obj, "product");
+		printf(" %04x:%04x\n", usb_vendor, usb_product);
+
+		uint8_t usb_bus = get_object_object_uint64(bus_data_obj, "bus");
+		uint8_t usb_bus_device = get_object_object_uint64(bus_data_obj, "device");
+		printf(L_LINE L_VAL "Bus %u, device %u\n", usb_bus, usb_bus_device);
 		break;
 	case DRM_BUS_PLATFORM:;
-		compatible_arr = json_object_object_get(data_obj, "compatible");
+		compatible_arr = json_object_object_get(device_data_obj, "compatible");
 		for (size_t i = 0; i < json_object_array_length(compatible_arr); i++) {
 			printf(" %s", json_object_get_string(
 				json_object_array_get_idx(compatible_arr, i)));
 		}
+		printf("\n");
+
+		fullname = json_object_get_string(json_object_object_get(bus_data_obj, "fullname"));
+		printf(L_LINE L_VAL "Bus: %s\n", fullname);
 		break;
 	case DRM_BUS_HOST1X:;
-		compatible_arr = json_object_object_get(data_obj, "compatible");
+		compatible_arr = json_object_object_get(device_data_obj, "compatible");
 		for (size_t i = 0; i < json_object_array_length(compatible_arr); i++) {
 			printf(" %s", json_object_get_string(
 				json_object_array_get_idx(compatible_arr, i)));
 		}
+		printf("\n");
+
+		fullname = json_object_get_string(json_object_object_get(bus_data_obj, "fullname"));
+		printf(L_LINE L_VAL "Bus: %s\n", fullname);
 		break;
+#if HAVE_FAUX_BUS
+	case DRM_BUS_FAUX:;
+		printf("\n");
+		const char *faux_name = json_object_get_string(
+			json_object_object_get(bus_data_obj, "name"));
+		printf(L_LINE L_VAL "Bus: %s\n", faux_name);
+		break;
+#endif
+	default:
+		printf("\n");
 	}
-	printf("\n");
 
 	printf(L_LINE L_LAST "Available nodes: ");
 	print_available_nodes(available_nodes);
@@ -459,6 +498,43 @@ static void print_size_hints(struct json_object *arr, const char *prefix)
 	}
 }
 
+static void print_edid(struct json_object *obj, const char *prefix)
+{
+#if HAVE_LIBDISPLAY_INFO
+	const char *str = json_object_get_string(obj);
+	if (str == NULL) {
+		return;
+	}
+
+	uint8_t *raw;
+	size_t size;
+	if (!base64_decode(str, &raw, &size)) {
+		fprintf(stderr, "Invalid base64-encoded EDID\n");
+		return;
+	}
+
+	struct di_info *info = di_info_parse_edid(raw, size);
+	free(raw);
+	if (info == NULL) {
+		fprintf(stderr, "Failed to parse EDID\n");
+		return;
+	}
+
+	char *make = di_info_get_make(info);
+	char *model = di_info_get_model(info);
+	char *serial = di_info_get_serial(info);
+	di_info_destroy(info);
+
+	printf("%s" L_VAL "Manufacturer: %s\n", prefix, make ? make : "Unknown");
+	printf("%s" L_VAL "Model: %s\n", prefix, model ? model : "Unknown");
+	printf("%s" L_LAST "Serial: %s\n", prefix, serial ? serial : "Unknown");
+
+	free(make);
+	free(model);
+	free(serial);
+#endif
+}
+
 static void print_fb(struct json_object *obj, const char *prefix)
 {
 	uint32_t id = get_object_object_uint64(obj, "id");
@@ -513,6 +589,24 @@ static void print_fb(struct json_object *obj, const char *prefix)
 	}
 }
 
+static void print_tile(struct json_object *obj, const char *prefix)
+{
+	int group_id = json_object_get_int(json_object_object_get(obj, "group_id"));
+	bool is_single_monitor = json_object_get_boolean(json_object_object_get(obj, "group_id"));
+	int h_num = json_object_get_int(json_object_object_get(obj, "h_num"));
+	int v_num = json_object_get_int(json_object_object_get(obj, "v_num"));
+	int h_loc = json_object_get_int(json_object_object_get(obj, "h_loc"));
+	int v_loc = json_object_get_int(json_object_object_get(obj, "v_loc"));
+	int h_size = json_object_get_int(json_object_object_get(obj, "h_size"));
+	int v_size = json_object_get_int(json_object_object_get(obj, "v_size"));
+
+	printf("%s" L_VAL "Group ID: %d\n", prefix, group_id);
+	printf("%s" L_VAL "Single monitor: %s\n", prefix, is_single_monitor ? "yes" : "no");
+	printf("%s" L_VAL "Number of tiles: %d×%d\n", prefix, h_num, v_num);
+	printf("%s" L_VAL "Tile location: %d, %d\n", prefix, h_loc, v_loc);
+	printf("%s" L_LAST "Tile size: %d×%d\n", prefix, h_size, v_size);
+}
+
 static void print_properties(struct json_object *obj, const char *prefix)
 {
 	printf("%s" L_LAST "Properties\n", prefix);
@@ -615,6 +709,10 @@ static void print_properties(struct json_object *obj, const char *prefix)
 				print_hdr_output_metadata(data_obj, sub_prefix);
 			else if (strcmp(prop_name, "SIZE_HINTS") == 0)
 				print_size_hints(data_obj, sub_prefix);
+			else if (strcmp(prop_name, "EDID") == 0)
+				print_edid(data_obj, sub_prefix);
+			else if (strcmp(prop_name, "TILE") == 0)
+				print_tile(data_obj, sub_prefix);
 			break;
 		case DRM_MODE_PROP_BITMASK:
 			printf("bitmask {");
